#
# Copyright (c) 2014 Hyung-Suk Kim, Jorge Herrera, Blair Kaneshiro
# All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a 
# copy of this software and associated documentation files (the 
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#   
# The authors encourage users to include this copyright notice,
# and to let us know that you are using the Software Any person wishing to 
# distribute modifications to the Software is encouraged to send the 
# modifications to the original authors so that they can be incorporated 
# into the canonical version.
# 
# The Software is provided "as is", WITHOUT ANY WARRANTY, express or implied,
# including but not limited to the warranties of MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE and NONINFRINGEMENT.  In no event shall the authors
# or copyright holders by liable for any claim, damages, or other liability,
# whether in an actino of a contract, tort or otherwise, arising from, out of
# or in connection with the Software or the use or other dealings in the 
# software.
#

# load sources
source('contourFunctions.R')

### NOTE - setup the directories where the actual data files are here
csvDir = '../../Final Dataset/csv/'
onsetDir = '../../Final Dataset/onset/'
groundTruthDir = '../../Final Dataset/ground_truth/onset/'

### Create match library
# load data
songData = read.csv(paste(csvDir, 'songs.csv', sep=""), stringsAsFactors=FALSE)

durDict={}
pitchDict={}
for( i in 1:nrow(songData) ) {
  
  filename = songData$filename[i]
  songTitle = songData$song_title[i]
  
  # load onset file
  filePath = paste(groundTruthDir, filename, '.onset', sep="")
  gtOnset = loadQueryFile(filePath)
  
  # load release file
  filePath = paste(groundTruthDir, filename, '.release', sep="")
  gtRelease = loadQueryFile(filePath)
  
  durDict[[songTitle]] = getDurationString(gtOnset, gtRelease)
  
  # load note_on file
  filePath = paste(groundTruthDir, filename, '.note_on', sep="")
  gtNote = loadQueryFile(filePath)
  
  pitchDict[[songTitle]] = getMelString(gtNote, 0)
}

### Match against dictionary
metadata = read.csv(paste(onsetDir, '0000_metadata.csv', sep=""), stringsAsFactors=FALSE, sep=",")
taskfiles = read.csv(paste(onsetDir, '0000.tasknumbers', sep=""), header=FALSE, sep = ",")
taskCount = length(taskfiles)

for(qi in 1:taskCount) {
  
  answer = trim.leading(metadata$song_title[qi])
  
  # process duration(onset) data
  zeroedFile = formatC(unlist(taskfiles[qi]), width=4, flag="0")
  filePath = paste(onsetDir, zeroedFile, '.onset', sep="")
  
  if( file.exists(filePath) ) { # check if file exists. empty files were removed
    onset = loadQueryFile(filePath)
    filePath = paste(onsetDir, zeroedFile, '.release', sep="")
    release = loadQueryFile(filePath)
    qString = getDurationString(onset, release)
    
    durResults = getScores(qString, durDict)
    
    # find if 
    index = match(answer, names(durResults))
    metadata$dur_best1[qi] = (index == 1)
    metadata$dur_best5[qi] = (index < 6)
    metadata$dur_best10[qi] = (index < 11)
    metadata$dur_best15[qi] = (index < 16)
  
    # process melody(pitch) data
    filePath = paste(onsetDir, zeroedFile, '.y_onset', sep="")
    yPos = loadQueryFile(filePath)
    
    melString = getMelString(yPos, 0.2)
    
    melResults = getScores(melString, pitchDict)
    
    index = match(answer, names(melResults))
    metadata$mel_best1[qi] = (index == 1)
    metadata$mel_best5[qi] = (index < 6)
    metadata$mel_best10[qi] = (index < 11)
    metadata$mel_best15[qi] = (index < 16)
    
    # use both results
    bothResults = mergeDistances(durResults, melResults, pow=1)
    
    index = match(answer, names(bothResults))
    metadata$both_best1[qi] = (index == 1)
    metadata$both_best5[qi] = (index < 6)
    metadata$both_best10[qi] = (index < 11)
    metadata$both_best15[qi] = (index < 16)
  }
}

### print accuracies
print(sum(metadata$dur_best1)/taskCount)
print(sum(metadata$dur_best5)/taskCount)
print(sum(metadata$dur_best10)/taskCount)
print(sum(metadata$dur_best15)/taskCount)

print(sum(metadata$mel_best1)/taskCount)
print(sum(metadata$mel_best5)/taskCount)
print(sum(metadata$mel_best10)/taskCount)
print(sum(metadata$mel_best15)/taskCount)

print(sum(metadata$both_best1)/taskCount)
print(sum(metadata$both_best5)/taskCount)
print(sum(metadata$both_best10)/taskCount)
print(sum(metadata$both_best15)/taskCount)

### McNemar test, 
# to asses the statistical significance of the accuracy improvement by adding the pitch
tmp1 = metadata$both_best1 - metadata$dur_best1;
tmp5 = metadata$both_best5 - metadata$dur_best5;
tmp10 = metadata$both_best10 - metadata$dur_best10;

contingency1 = matrix(c(sum(tmp1==0 & metadata$dur_best1), sum(tmp1==1), sum(tmp1==-1), sum(tmp1==0 & !metadata$dur_best1)), nrow=2);
contingency5 = matrix(c(sum(tmp5==0 & metadata$dur_best5), sum(tmp5==1), sum(tmp5==-1), sum(tmp5==0 & !metadata$dur_best5)), nrow=2);
contingency10 = matrix(c(sum(tmp10==0 & metadata$dur_best10), sum(tmp10==1), sum(tmp10==-1), sum(tmp10==0 & !metadata$dur_best10)), nrow=2);

# sum(sum(contingency1))
# sum(sum(contingency5))
# sum(sum(contingency10))

continuity_correction = TRUE
mcnemar.test(contingency1, correct=continuity_correction)
mcnemar.test(contingency5, correct=continuity_correction)
mcnemar.test(contingency10, correct=continuity_correction)
